@@ -5,6 +5,7 @@ import websockets
 from .common import get_base_url, get_credentials
 from .entity import Account, Entity
 from . import polygon
+import logging
 
 
 class StreamConn(object):
@@ -13,9 +14,11 @@ class StreamConn(object):
         base_url = re.sub(r'^http', 'ws', base_url or get_base_url())
         self._endpoint = base_url + '/stream'
         self._handlers = {}
+        self._handler_symbols = {}
         self._base_url = base_url
         self._ws = None
         self.polygon = None
+        self.loop = asyncio.get_event_loop()
 
     async def _connect(self):
         ws = await websockets.connect(self._endpoint)
@@ -58,14 +61,15 @@ class StreamConn(object):
             await ws.close()
             self._ws = None
 
-    async def _ensure_nats(self):
+    async def _ensure_polygon(self):
         if self.polygon is not None:
             return
         key_id = self._key_id
         if 'staging' in self._base_url:
             key_id += '-staging'
-        self.polygon = polygon.Stream(key_id)
-        self.polygon.register(r'.*', self._dispatch_nats)
+        self.polygon = polygon.StreamConn(key_id)
+        self.polygon._handlers = self._handlers
+        self.polygon._handler_symbols = self._handler_symbols
         await self.polygon.connect()
 
     async def _ensure_ws(self):
@@ -78,10 +82,10 @@ class StreamConn(object):
         If the necessary connection isn't open yet, it opens now.
         '''
         ws_channels = []
-        nats_channels = []
+        polygon_channels = []
         for c in channels:
             if c.startswith(('Q.', 'T.', 'A.', 'AM.',)):
-                nats_channels.append(c)
+                polygon_channels.append(c)
             else:
                 ws_channels.append(c)
 
@@ -94,20 +98,45 @@ class StreamConn(object):
                 }
             }))
 
-        if len(nats_channels) > 0:
-            await self._ensure_nats()
-            await self.polygon.subscribe(nats_channels)
+        if len(polygon_channels) > 0:
+            await self._ensure_polygon()
+            await self.polygon.subscribe(polygon_channels)
+
+    async def unsubscribe(self, channels):
+        '''Handle un-subscribing from channels.
+        '''
+        if not self._ws:
+            return
+
+        ws_channels = []
+        polygon_channels = []
+        for c in channels:
+            if c.startswith(('Q.', 'T.', 'A.', 'AM.',)):
+                polygon_channels.append(c)
+            else:
+                ws_channels.append(c)
+
+        if len(ws_channels) > 0:
+            # Currently our streams don't support unsubscribe
+            # not as useful with our feeds
+            pass
+
+        if len(polygon_channels) > 0:
+            await self.polygon.unsubscribe(polygon_channels)
 
     def run(self, initial_channels=[]):
         '''Run forever and block until exception is rasised.
         initial_channels is the channels to start with.
         '''
-        loop = asyncio.get_event_loop()
+        loop = self.loop
         try:
             loop.run_until_complete(self.subscribe(initial_channels))
             loop.run_forever()
+        except KeyboardInterrupt:
+            logging.info("Exiting on Interrupt")
         finally:
             loop.run_until_complete(self.close())
+            loop.close()
 
     async def close(self):
         '''Close any of open connections'''
@@ -121,32 +150,33 @@ class StreamConn(object):
             return Account(msg)
         return Entity(msg)
 
-    async def _dispatch_nats(self, conn, subject, data):
-        for pat, handler in self._handlers.items():
-            if pat.match(subject):
-                await handler(self, subject, data)
-
     async def _dispatch(self, channel, msg):
         for pat, handler in self._handlers.items():
             if pat.match(channel):
                 ent = self._cast(channel, msg['data'])
                 await handler(self, channel, ent)
 
-    def on(self, channel_pat):
+    def on(self, channel_pat, symbols=None):
         def decorator(func):
-            self.register(channel_pat, func)
+            self.register(channel_pat, func, symbols)
             return func
 
         return decorator
 
-    def register(self, channel_pat, func):
+    def register(self, channel_pat, func, symbols=None):
         if not asyncio.iscoroutinefunction(func):
             raise ValueError('handler must be a coroutine function')
         if isinstance(channel_pat, str):
             channel_pat = re.compile(channel_pat)
         self._handlers[channel_pat] = func
+        self._handler_symbols[func] = symbols
+        if self.polygon:
+            self.polygon.register(channel_pat, func, symbols)
 
     def deregister(self, channel_pat):
         if isinstance(channel_pat, str):
             channel_pat = re.compile(channel_pat)
+        self._handler_symbols.pop(self._handlers[channel_pat], None)
         del self._handlers[channel_pat]
+        if self.polygon:
+            self.polygon.deregister(channel_pat)
